// -*- C++ -*-
/*!
 * @file  CANController.cpp
 * @brief CANControllerComponent
 * @date $Date$
 *
 * $Id$
 */

#include "CANController.h"

// Module specification
// <rtc-template block="module_spec">
static const char* cancontroller_spec[] =
  {
    "implementation_id", "CANController",
    "type_name",         "CANController",
    "description",       "CANControllerComponent",
    "version",           "1.0.0",
    "vendor",            "ToshihiroMiura_Shibaura.HRI",
    "category",          "CANComponent",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "10",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
	"conf.ResetFilter.SetMode", "0",

    "conf.SetFilter0.SetMode", "1",
    "conf.SetFilter0.ID_From", "2",
    "conf.SetFilter0.ID_To", "5",

    "conf.SetFilter1.SetMode", "1",
    "conf.SetFilter1.ID_From", "0",
    "conf.SetFilter1.ID_To", "5",

    "conf.SetFilter2.SetMode", "1",
    "conf.SetFilter2.ID_From", "5",
    "conf.SetFilter2.ID_To", "10",

    ""
  };
// </rtc-template>

CANController::CANController(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_TxChannel1In("TxChannel1", m_TxChannel1),
    m_TxChannel2In("TxChannel2", m_TxChannel2),
    m_TxChannelAllIn("TxChannelAll", m_TxChannelAll),
    m_RxChannel1Out("RxChannel1", m_RxChannel1),
    m_RxChannel2Out("RxChannel2", m_RxChannel2),
    m_RxChannelAllOut("RxChannelAll", m_RxChannelAll),
    
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("TxChannel1", m_TxChannel1In);
  registerInPort("TxChannel2", m_TxChannel2In);
  registerInPort("TxChannelAll", m_TxChannelAllIn);
  
  // Set OutPort buffer
  registerOutPort("RxChannel1", m_RxChannel1Out);
  registerOutPort("RxChannel2", m_RxChannel2Out);
  registerOutPort("RxChannelAll", m_RxChannelAllOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

	m_pCAN = new CCANFunctionsXL(this);
}

CANController::~CANController()
{
	delete m_pCAN;
}


RTC::ReturnCode_t CANController::onInitialize()
{
	// <rtc-template block="bind_config">
	// Bind variables and configuration variable
	std::cout << "--------------------------------------------------" << std::endl;
	std::cout << " Module Name: CANControllerComponent" << std::endl;
	std::cout << " Vendor  : Toshihiro MIURA" << std::endl;
	std::cout << "           SIT Mizukawa Lab." << std::endl;
	std::cout << "--------------------------------------------------" << std::endl;
	std::cout << "***** <<Channel Setting>> ******" << std::endl;
	m_bSetFilterFlag = false;
	bindParameter("SetMode", m_SetMode, "0");
	bindParameter("ID_From", m_FromID, "2");
	bindParameter("ID_To", m_ToID, "5");
	if (m_pCAN->Initialize())
	{
		std::cout << "<ERROR> no HW!" << std::endl << "You need two" << std::endl << "CANcabs/CANpiggy's";
		std::cout << "********************************" << std::endl;
		return RTC::RTC_ERROR;
	}
	std::cout << "********************************" << std::endl;
	// </rtc-template>
	return RTC::RTC_OK;
}


RTC::ReturnCode_t CANController::onActivated(RTC::UniqueId ec_id)
{
	XLstatus xlStatus;
	unsigned long Baudrate[5] = {33333, 100000, 125000, 500000, 1000000};	// Vector JapanCANCardXL33.3[kbps],100[kbps],125[kbps],500[kbps],1[Mbps]T|[g
	xlStatus = m_pCAN->ConnectBus(Baudrate[4]);							// Bardrateݒ肵܂D(1Mbps)
	if (!xlStatus)
	{
		std::cout << "Successfully Connect!" << std::endl;
		return RTC::RTC_OK;
	}
	else
	{
		std::cout << "Error Connect!!" << std::endl;
		return RTC::RTC_ERROR;
	}
}



RTC::ReturnCode_t CANController::onDeactivated(RTC::UniqueId ec_id)
{
	XLstatus xlStatus;
	xlStatus = m_pCAN->DisconnectBus();
	if (!xlStatus) 
	{
		std::cout << "Successfully Disconnect!" << std::endl;
		return RTC::RTC_OK;
	}
	else
	{
		std::cout << "Error Disconnect!!" << std::endl;
		return RTC::RTC_ERROR;
	}
}


RTC::ReturnCode_t CANController::onExecute(RTC::UniqueId ec_id)
{
	XLevent xlEventTmp;
	if (m_SetMode != m_bSetFilterFlag)
	{
		if(m_SetMode)
		{
			m_pCAN->SetFilter(m_FromID, m_ToID);
			m_bSetFilterFlag = true;
			m_ulFromIDLog = m_FromID;
			m_ulToIDLog = m_ToID;
			std::cout << "SetFilter(" << m_FromID << "`" << m_ToID << ")" << std::endl;
		}
		else
		{
			m_bSetFilterFlag = false;
			m_pCAN->ResetFilter();
			std::cout << "ResetFilter!" << std::endl;
		}
	}
	else if (m_SetMode)
	{
		if (m_FromID != m_ulFromIDLog)
		{
			m_pCAN->SetFilter(m_FromID, m_ToID);
			m_bSetFilterFlag = true;
			m_ulFromIDLog = m_FromID;
			m_ulToIDLog = m_ToID;
			std::cout << "SetFilter(" << m_FromID << "`" << m_ToID << ")" << std::endl;
		}
		else if (m_ToID != m_ulToIDLog)
		{
			m_pCAN->SetFilter(m_FromID, m_ToID);
			m_bSetFilterFlag = true;
			m_ulFromIDLog = m_FromID;
			m_ulToIDLog = m_ToID;
			std::cout << "SetFilter(" << m_FromID << "`" << m_ToID << ")" << std::endl;
		}
	}

	if (m_TxChannel1In.isNew())
	{
		m_TxChannel1In.read();
		//std::cout << "Channel1 (SendMessage)" << std::endl;	
		xlEventTmp = ClassifyTxMessage(m_TxChannel1);
		m_pCAN->SendMessage(xlEventTmp, 0);							// ݒ肵CAN Message𑗐M
		//std::cout << std::endl;
	}
	if (m_TxChannel2In.isNew())
	{
		m_TxChannel2In.read();
		//std::cout << "Channel2 (SendMessage)" << std::endl;
		xlEventTmp = ClassifyTxMessage(m_TxChannel2);
		m_pCAN->SendMessage(xlEventTmp, 1); // ݒ肵CAN Message𑗐M
		//std::cout << std::endl;
	}
	if (m_TxChannelAllIn.isNew())
	{
		m_TxChannelAllIn.read();
		//std::cout << "ChannelAll (SendMessage)" << std::endl;
		xlEventTmp = ClassifyTxMessage(m_TxChannelAll);
		m_pCAN->SendMessage(xlEventTmp, 0); // ݒ肵CAN MessageChannel1֑M
		m_pCAN->SendMessage(xlEventTmp, 1); // ݒ肵CAN MessageChannel2֑M
		//std::cout << std::endl;
	}
  return RTC::RTC_OK;
}


TimedOctetSeq CANController::ComposeRxMessage(XLevent xlEventRx)
{
	TimedOctetSeq RxMessage;
	unsigned long ulID;
	int i;
	XLuint64 xl64TimeStamp;

	RxMessage.data.length(xlEventRx.tagData.msg.dlc + 14);
	ulID = xlEventRx.tagData.msg.id;
	for (i = 0; i < 4; i++)										// ID8[Bit]ɕ
	{
		RxMessage.data[i] = (BYTE)ulID;
		ulID = ulID >> 8;
	}
	RxMessage.data[i] = xlEventRx.tagData.msg.dlc;
	for (i++; i < xlEventRx.tagData.msg.dlc + 5; i++)
		RxMessage.data[i] = xlEventRx.tagData.msg.data[i - 5];
	if (xlEventRx.portHandle == 0)									// RxMessagȅꍇ
		RxMessage.data[i] = (BYTE)xlEventRx.chanIndex;
	else
		RxMessage.data[i] = (BYTE)(xlEventRx.chanIndex + 16);	// TxMessagȅꍇ
	xl64TimeStamp = xlEventRx.timeStamp;
	for (i++; i < xlEventRx.tagData.msg.dlc + 14; i++)
	{
		RxMessage.data[i] = (BYTE)xl64TimeStamp;
		xl64TimeStamp = xl64TimeStamp >> 8;
	}
	return RxMessage;
}

XLevent CANController::ClassifyTxMessage(TimedOctetSeq TxChannel)
{
	int i;
	XLevent xlEventTx;
	memset(&xlEventTx, 0, sizeof(xlEventTx));
	xlEventTx.tag = XL_TRANSMIT_MSG;
	
	for(i = 0; i < 4; i++)
	{
		xlEventTx.tagData.msg.id = xlEventTx.tagData.msg.id + TxChannel.data[3 - i];
		if (i < 3)
			xlEventTx.tagData.msg.id = xlEventTx.tagData.msg.id << 8;
	}
	//std::cout << "ID : " << xlEventTx.tagData.msg.id << "  ";
	xlEventTx.tagData.msg.dlc = (unsigned short)TxChannel.data[4];
	//std::cout << "Data[" << xlEventTx.tagData.msg.dlc << "Byte] : ";
	xlEventTx.tagData.msg.flags   = 0;
	for (int i = 0; i < xlEventTx.tagData.msg.dlc; i++)
	{
		xlEventTx.tagData.msg.data[i] = (unsigned char)TxChannel.data[i + 5];
	//	std::cout << (int)TxChannel.data[i + 5] << " ";
	}
	return xlEventTx;
}


void CANController::OnReceive(XLevent xlEvent)
{
	//std::cout << xlGetEventString(&xlEvent) << std::endl;
	m_RxChannelAll = ComposeRxMessage(xlEvent);
	if (xlEvent.chanIndex == m_pCAN->m_uiChannel1)
	{
		m_RxChannel1 = m_RxChannelAll;
		m_RxChannel1Out.write();
	}
	else if (xlEvent.chanIndex == m_pCAN->m_uiChannel2)
	{
		m_RxChannel2 = m_RxChannelAll;
		m_RxChannel2Out.write();
	}
	m_RxChannelAllOut.write();
}


extern "C"
{
  void CANControllerInit(RTC::Manager* manager)
  {
    RTC::Properties profile(cancontroller_spec);
    manager->registerFactory(profile,
                             RTC::Create<CANController>,
                             RTC::Delete<CANController>);
  }
};
