#include "CANFunctionsXL.h"
#include <iostream>

//////////////////////////////////////////////////////////////////////
// Construction
//////////////////////////////////////////////////////////////////////
CCANFunctionsXL::CCANFunctionsXL(CReceiveEvent *pReceiveEvent)
{
	m_xlChannelMask[CHAN01] = 0;									// Channel1̐ݒ
	m_xlChannelMask[CHAN02] = 0;									// Channel2̐ݒ
	m_pReceiveEvent = pReceiveEvent;								// CANMessageM̃Cxgݒ
}


//////////////////////////////////////////////////////////////////////
// Destruction
//////////////////////////////////////////////////////////////////////
CCANFunctionsXL::~CCANFunctionsXL(void)
{
	CloseHandle(m_hMsgEvent);
	DeleteReceiveThread();
}



////////////////////////////////////////////////////////////////////////////
//! Initialize
//! Open the driver, get the channelmasks and create the RX thread.
////////////////////////////////////////////////////////////////////////////
XLstatus CCANFunctionsXL::Initialize(void)
{
	XLstatus xlStatus = XL_ERROR;
	
	xlStatus = xlOpenDriver();
	if (xlStatus != XL_SUCCESS)
		return xlStatus;
	/*--- Get/Set the application within VHWConf ---*/
	xlStatus = GetChannelMask();
	if ((xlStatus) || (m_xlChannelMask[CHAN01] == 0) || (m_xlChannelMask[CHAN02] == 0))
		return XL_ERROR;
	/*--- Open ONE Port for both channels ---*/
	xlStatus = CANInitialize();
	if (xlStatus)
		return xlStatus;
	return xlStatus;
}

////////////////////////////////////////////////////////////////////////////
//! GetChannelMask
//! parse the registry to get the channelmask
//////////////////////////////////////////////////////////////////////////// 
XLstatus CCANFunctionsXL::GetChannelMask(void)
{
	XLstatus        xlStatus = XL_ERROR;
	/*--- default values ---*/
	unsigned int  hwType     = 0;
	unsigned int  hwIndex    = 0;
	unsigned int  hwChannel  = 0;
	unsigned int  appChannel = 0;
	unsigned int  busType    = XL_BUS_TYPE_CAN;   
	unsigned int  i; 
	XLdriverConfig xlDrvConfig;
	/*--- Check for hardware ---*/
	xlStatus = xlGetDriverConfig(&xlDrvConfig);
	if (xlStatus)
		return xlStatus;
	/*--- we check only if there is an application registered or not. ---*/
	xlStatus = xlGetApplConfig("CANControllerComponent", CHAN01, &hwType, &hwIndex, &hwChannel, busType);
	/*--- Set the params into registry (default values...!) ---*/
	if (xlStatus)
	{
		for (i = 0; i < xlDrvConfig.channelCount; i++)
		{
			/*--- We search not the first CAN cads ---*/
			if ((xlDrvConfig.channel[i].channelBusCapabilities & XL_BUS_ACTIVE_CAP_CAN) && (appChannel < 2))	// Channelςꍇ͂̏ꏊCD
			{
				hwType = xlDrvConfig.channel[i].hwType;
				hwIndex = xlDrvConfig.channel[i].hwIndex;
				hwChannel = xlDrvConfig.channel[i].hwChannel;
				xlStatus = xlSetApplConfig(						// Registration of Application with default settings
											"CANControllerComponent",		// Application Name
											appChannel,			// Application channel 0 or 1
											hwType,				// hwType  (CANcardXL...)    
											hwIndex,			// Index of hardware (slot) (0,1,...)
											hwChannel,			// Index of channel (connector) (0,1,...)
											busType);			// the application is for CAN.
				m_xlChannelMask[appChannel] = xlGetChannelMask(hwType, hwIndex, hwChannel);
				std::cout <<  xlDrvConfig.channel[i].name << std::endl;
				appChannel++;
			}
		}
	}
	else
	{
		m_xlChannelMask[CHAN01] = xlGetChannelMask(hwType, hwIndex, hwChannel);
		for (i=0; i < xlDrvConfig.channelCount; i++)
			if ( xlDrvConfig.channel[i].channelMask == m_xlChannelMask[CHAN01])
			{
				std::cout << "Channel1: " << xlDrvConfig.channel[i].name << std::endl;
				m_uiChannel1 = i;
			}
		xlStatus = xlGetApplConfig("CANControllerComponent", CHAN02, &hwType, &hwIndex, &hwChannel, busType);	// get the second channel 
		if (xlStatus)
			return xlStatus;
		m_xlChannelMask[CHAN02] = xlGetChannelMask(hwType, hwIndex, hwChannel);
		for (i=0; i < xlDrvConfig.channelCount; i++)
			if ( xlDrvConfig.channel[i].channelMask == m_xlChannelMask[CHAN02])
			{
				std::cout << "Channel2: " << xlDrvConfig.channel[i].name << std::endl;
				m_uiChannel2 = i;
			}
	}
	return xlStatus;
}

////////////////////////////////////////////////////////////////////////////
//! CANInitialize
//! xlCANcontrol use ONE port for both channels.
//////////////////////////////////////////////////////////////////////////// 
XLstatus CCANFunctionsXL::CANInitialize(void)
{
	XLstatus         xlStatus = XL_ERROR;
	XLaccess         xlPermissionMask;
	/*--- Open ONE port for both channels ---*/
	// calculate the channelMask for both channel 
	m_xlChannelMask_both = m_xlChannelMask[CHAN01] + m_xlChannelMask[CHAN02];
	xlPermissionMask = m_xlChannelMask_both;
	xlStatus = xlOpenPort(&m_xlPortHandle, "CANControllerComponent", m_xlChannelMask_both, &xlPermissionMask, 256, XL_INTERFACE_VERSION, XL_BUS_TYPE_CAN); 
	if (m_xlPortHandle == XL_INVALID_PORTHANDLE)
		return XL_ERROR;
	if (xlStatus == XL_ERR_INVALID_ACCESS)
		return xlStatus;
	return xlStatus;
}


////////////////////////////////////////////////////////////////////////////
//! CreateReceiveThread
//! 
////////////////////////////////////////////////////////////////////////////
XLstatus CCANFunctionsXL::CreateReceiveThread(void)
{
	XLstatus      xlStatus = XL_ERROR;
	DWORD         ThreadId=0;
	if (m_xlPortHandle != XL_INVALID_PORTHANDLE)
	{
		xlStatus = xlSetNotification(m_xlPortHandle, &m_hMsgEvent, 1);
		m_isReceiving = true;
		m_hReceiveThread = CreateThread(0, 0x1000, ReceiveCANThread, this, 0, &ThreadId);
	}
	return xlStatus;
}

////////////////////////////////////////////////////////////////////////////
//! DeleteReceiveThread
//! 
////////////////////////////////////////////////////////////////////////////
XLstatus CCANFunctionsXL::DeleteReceiveThread(void)
{
	XLstatus xlStatus = XL_ERROR;
	m_isReceiving = false;
	CloseHandle(m_hReceiveThread);
	return xlStatus;
}

////////////////////////////////////////////////////////////////////////////
//! ConnectBus
//! 
////////////////////////////////////////////////////////////////////////////
XLstatus CCANFunctionsXL::ConnectBus(unsigned long Baudrate)
{
	XLstatus      xlStatus = XL_ERROR;

	xlStatus = CreateReceiveThread();
	//if (xlStatus)
	//	return xlStatus;

	xlStatus = xlCanSetChannelBitrate(m_xlPortHandle, m_xlChannelMask_both, Baudrate);
	xlStatus = xlActivateChannel(m_xlPortHandle, m_xlChannelMask_both, XL_BUS_TYPE_CAN, XL_ACTIVATE_RESET_CLOCK);
	return xlStatus;
}

////////////////////////////////////////////////////////////////////////////
//! DisconnectBus
//! Deactivate the channel
////////////////////////////////////////////////////////////////////////////
XLstatus CCANFunctionsXL::DisconnectBus(void)
{
	XLstatus      xlStatus = XL_ERROR;
	DeleteReceiveThread();
	xlStatus = xlDeactivateChannel(m_xlPortHandle, m_xlChannelMask_both);
	return xlStatus;
}

////////////////////////////////////////////////////////////////////////////
//! SendMessage
//! transmit a CAN message to the selected channel with the give values.
////////////////////////////////////////////////////////////////////////////
XLstatus CCANFunctionsXL::SendMessage(XLevent xlEvent, int channel)
{
	XLstatus      xlStatus;
	unsigned int uiMessageCount = 1;
	xlStatus = xlCanTransmit(m_xlPortHandle, m_xlChannelMask[channel], &uiMessageCount, &xlEvent);
	return xlStatus;
}


////////////////////////////////////////////////////////////////////////////
//! ResetFilter
//! Reset the acceptancefilter
////////////////////////////////////////////////////////////////////////////
XLstatus CCANFunctionsXL::ResetFilter(void)
{
	XLstatus      xlStatus;
	xlStatus = xlCanResetAcceptance(m_xlPortHandle, m_xlChannelMask_both, XL_CAN_STD);
	return xlStatus;
}

////////////////////////////////////////////////////////////////////////////
//! SetFilter
//! Reset the acceptancefilter
//!
////////////////////////////////////////////////////////////////////////////
XLstatus CCANFunctionsXL::SetFilter(unsigned long first_id, unsigned long last_id)
{
	XLstatus      xlStatus;
	xlStatus = xlCanSetChannelAcceptance(m_xlPortHandle, m_xlChannelMask_both, 0xFFF, 0xFFF, XL_CAN_STD);
	xlStatus = xlCanAddAcceptanceRange(m_xlPortHandle, m_xlChannelMask_both, first_id, last_id);
	return xlStatus;
}

////////////////////////////////////////////////////////////////////////////
//! FireOnReceive
//! 
////////////////////////////////////////////////////////////////////////////
void CCANFunctionsXL::FireOnReceive(XLevent xlEvent)
{
	if(m_pReceiveEvent != NULL)										// Cxg̏
		m_pReceiveEvent->OnReceive(xlEvent);
}

////////////////////////////////////////////////////////////////////////////
//! ReceiveCANThread
//! 
////////////////////////////////////////////////////////////////////////////
DWORD WINAPI CCANFunctionsXL::ReceiveCANThread(LPVOID pParam)
{
	CCANFunctionsXL *pInst = (CCANFunctionsXL *) pParam;			// CX^X
	XLstatus xlStatus;
	unsigned int uiMsgsrx = 1;
	XLevent xlEvent;
	//std::cout << "Thread Create!" << std::endl;
	while (pInst->m_isReceiving)
	{
		WaitForSingleObject(pInst->m_hMsgEvent, 10);
		xlStatus = XL_SUCCESS;
		while (!xlStatus)
		{
			uiMsgsrx = 1;
			xlStatus = xlReceive(pInst->m_xlPortHandle, &uiMsgsrx, &xlEvent);
			if (xlStatus != XL_ERR_QUEUE_IS_EMPTY)
				pInst->FireOnReceive(xlEvent);
		}
	}
	//std::cout << "Thread Close!" << std::endl;
	return 0;
}