// -*- C++ -*-
/*!
 * @file  ClassifyMessage.cpp
 * @brief ClassifyMessage
 * @date $Date$
 *
 * $Id$
 */

#include "ClassifyMessage.h"

// Module specification
// <rtc-template block="module_spec">
static const char* classifymessage_spec[] =
  {
    "implementation_id", "ClassifyMessage",
    "type_name",         "ClassifyMessage",
    "description",       "ClassifyMessage",
    "version",           "1.0.0",
    "vendor",            "ToshihiroMiura_Shibaura.HRI",
    "category",          "CANComponents",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    ""
  };
// </rtc-template>

ClassifyMessage::ClassifyMessage(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_RxMessageIn("RxMessage", m_RxMessage),
    m_SWOut("SW", m_SW),
    m_DACOut("DAC", m_DAC),
    
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("RxMessage", m_RxMessageIn);
  
  // Set OutPort buffer
  registerOutPort("SW", m_SWOut);
  registerOutPort("DAC", m_DACOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

}

ClassifyMessage::~ClassifyMessage()
{
}


CANRxMessageFrame ClassifyMessage::ClassifyRxMessage(TimedOctetSeq RxMessage)
{
	int i, iSize;
	CANRxMessageFrame CANRxMessage;
	memset(&CANRxMessage, 0, sizeof(CANRxMessage));
	iSize = RxMessage.data.length();

	for (i = 0; i < 4; i++)
	{
		CANRxMessage.ID = CANRxMessage.ID + (unsigned long)RxMessage.data[3 - i];
		if (i < 3)
			CANRxMessage.ID = CANRxMessage.ID << 8;
	}
	CANRxMessage.DLC = (BYTE)RxMessage.data[i];
	for (i++; i < CANRxMessage.DLC + 5; i++)
		CANRxMessage.Data[i - 5].ByteAccess = (BYTE)RxMessage.data[i];
	if (iSize > CANRxMessage.DLC + 5)
		CANRxMessage.Type.ByteAccess = (BYTE)RxMessage.data[i];
	else
		CANRxMessage.Type.ByteAccess = 0;
	if (iSize == CANRxMessage.DLC +14)
	{
		for (i = (CANRxMessage.DLC + 13); i > (CANRxMessage.DLC + 5); i--)
		{
			CANRxMessage.TimeStamp = CANRxMessage.TimeStamp + RxMessage.data[i];
			if (i > CANRxMessage.DLC + 6)
				CANRxMessage.TimeStamp = CANRxMessage.TimeStamp << 8;
		}
	}
	else
		CANRxMessage.TimeStamp = 0;
	return CANRxMessage;
}


RTC::ReturnCode_t ClassifyMessage::onInitialize()
{
	std::cout << "--------------------------------------------------" << std::endl;
	std::cout << " Module Name: ClassifyMessageComponent" << std::endl;
	std::cout << " Vendor  : Toshihiro MIURA" << std::endl;
	std::cout << "           SIT Mizukawa Lab." << std::endl;
	std::cout << "--------------------------------------------------" << std::endl;
	return RTC::RTC_OK;
}


RTC::ReturnCode_t ClassifyMessage::onExecute(RTC::UniqueId ec_id)
{
	if (m_RxMessageIn.isNew())
	{
		CANRxMessageFrame CANRxMessageTmp;
		memset(&CANRxMessageTmp, 0, sizeof(CANRxMessageTmp));
		m_RxMessageIn.read();
		CANRxMessageTmp = ClassifyRxMessage(m_RxMessage);
		switch (CANRxMessageTmp.ID)
		{
		case 2:
			m_SW.data = CANRxMessageTmp.Data[0].ByteAccess;
			m_SWOut.write();
			std::cout << "Switch: " << (int)CANRxMessageTmp.Data[0].BitAccess.B7 <<  ' ' << (int)CANRxMessageTmp.Data[0].BitAccess.B6 << ' ' << (int)CANRxMessageTmp.Data[0].BitAccess.B5 << ' ' << (int)CANRxMessageTmp.Data[0].BitAccess.B4 << ' ' << (int)CANRxMessageTmp.Data[0].BitAccess.B3 << ' ' << (int)CANRxMessageTmp.Data[0].BitAccess.B2 << ' ' << (int)CANRxMessageTmp.Data[0].BitAccess.B1 << ' ' << (int)CANRxMessageTmp.Data[0].BitAccess.B0 << std::endl;
			break;
		case 4:
			m_DAC.data = CANRxMessageTmp.Data[1].ByteAccess;
			m_DAC.data = (m_DAC.data << 8) + CANRxMessageTmp.Data[0].ByteAccess;
			m_DACOut.write();
			std::cout << "DAC: "<< m_DAC.data << std::endl;
			break;
		}
	}
  return RTC::RTC_OK;
}


extern "C"
{
 
  void ClassifyMessageInit(RTC::Manager* manager)
  {
    RTC::Properties profile(classifymessage_spec);
    manager->registerFactory(profile,
                             RTC::Create<ClassifyMessage>,
                             RTC::Delete<ClassifyMessage>);
  }
  
};
