// -*- C++ -*-
/*!
 * @file  ComposeMessage.cpp
 * @brief ComposeMessageComponent
 * @date $Date$
 *
 * $Id$
 */

#include "ComposeMessage.h"

// Module specification
// <rtc-template block="module_spec">
static const char* composemessage_spec[] =
  {
    "implementation_id", "ComposeMessage",
    "type_name",         "ComposeMessage",
    "description",       "ComposeMessageComponent",
    "version",           "1.0.0",
    "vendor",            "Toshihiro.MIURA_ShibauraHRI",
    "category",          "CANComp",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "10",
    "language",          "C++",
    "lang_type",         "compile",
    ""
  };
// </rtc-template>

ComposeMessage::ComposeMessage(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_LEDIn("LED8bit", m_LED),
    m_ADCIn("ADC", m_ADC),
    m_TxMessageOut("TxMessage", m_TxMessage),
    
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("LED8bit", m_LEDIn);
  registerInPort("ADC", m_ADCIn);
  
  // Set OutPort buffer
  registerOutPort("TxMessage", m_TxMessageOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

}

ComposeMessage::~ComposeMessage()
{
}

TimedOctetSeq ComposeMessage::ComposeTxMessage(CANTxMessageFrame CANTxMessage)
{
	TimedOctetSeq TxMessage;
	unsigned long ulID;
	int i;
	TxMessage.data.length(CANTxMessage.DLC + 5);
	ulID = CANTxMessage.ID;
	for(i= 0; i < 4; i++)
	{
		TxMessage.data[i] = (BYTE)ulID;
		ulID = ulID >> 8;
	}
	TxMessage.data[i] = CANTxMessage.DLC;
	for(i++; i < CANTxMessage.DLC + 5; i++)
		TxMessage.data[i] = CANTxMessage.Data[i - 5].ByteAccess;
	return TxMessage;
}

RTC::ReturnCode_t ComposeMessage::onInitialize()
{
	std::cout << "--------------------------------------------------" << std::endl;
	std::cout << " Module Name: ComposeMessageComponent" << std::endl;
	std::cout << " Vendor  : Toshihiro MIURA" << std::endl;
	std::cout << "           SIT Mizukawa Lab." << std::endl;
	std::cout << "--------------------------------------------------" << std::endl;
	return RTC::RTC_OK;
}

RTC::ReturnCode_t ComposeMessage::onExecute(RTC::UniqueId ec_id)
{
	CANTxMessageFrame CANTxMessageTmp;
	if (m_LEDIn.isNew())
	{
		m_LEDIn.read();
		memset(&CANTxMessageTmp, 0, sizeof(CANTxMessageTmp));
		CANTxMessageTmp.ID = 1;
		CANTxMessageTmp.DLC = 1;
		CANTxMessageTmp.Data[0].ByteAccess = m_LED.data;
		m_TxMessage = ComposeTxMessage(CANTxMessageTmp);
		m_TxMessageOut.write();
		std::cout << "LED : " << (int)CANTxMessageTmp.Data[0].BitAccess.B7 << ' ' << (int)CANTxMessageTmp.Data[0].BitAccess.B6 << ' ' << (int)CANTxMessageTmp.Data[0].BitAccess.B5 << ' ' << (int)CANTxMessageTmp.Data[0].BitAccess.B4 << ' ' << (int)CANTxMessageTmp.Data[0].BitAccess.B3 << ' ' << (int)CANTxMessageTmp.Data[0].BitAccess.B2 << ' ' << (int)CANTxMessageTmp.Data[0].BitAccess.B1 << ' ' << (int)CANTxMessageTmp.Data[0].BitAccess.B0 << std::endl;
	}
	if (m_ADCIn.isNew())
	{
		m_ADCIn.read();
		memset(&CANTxMessageTmp, 0, sizeof(CANTxMessageTmp));
		CANTxMessageTmp.ID = 3;
		CANTxMessageTmp.DLC = 2;
		CANTxMessageTmp.Data[0].ByteAccess = (BYTE)m_ADC.data;
		CANTxMessageTmp.Data[1].ByteAccess = (BYTE)(m_ADC.data >> 8);
		m_TxMessage = ComposeTxMessage(CANTxMessageTmp);
		m_TxMessageOut.write();
		std::cout << "ADC : " << m_ADC.data << std::endl;
	}
  return RTC::RTC_OK;
}


extern "C"
{
  void ComposeMessageInit(RTC::Manager* manager)
  {
    RTC::Properties profile(composemessage_spec);
    manager->registerFactory(profile,
                             RTC::Create<ComposeMessage>,
                             RTC::Delete<ComposeMessage>);
  }  
};
