// -*- C++ -*-
/*!
 * @file  Output.cpp
 * @brief OutputComponent
 * @date $Date$
 *
 * $Id$
 */

#include "Output.h"
#include <stdio.h>

// Module specification
// <rtc-template block="module_spec">
static const char* output_spec[] =
  {
    "implementation_id", "Output",
    "type_name",         "Output",
    "description",       "OutputComponent",
    "version",           "1.0.0",
    "vendor",            "Toshihiro.MIURA_ShibauraHRI",
    "category",          "TestComp",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "10",
    "language",          "C++",
    "lang_type",         "compile",
    ""
  };
// </rtc-template>

Output::Output(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_SWIn("LED", m_SW),
    m_DACIn("DAC", m_DAC),
    
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("SW", m_SWIn);
  registerInPort("DAC", m_DACIn);
  
  // Set OutPort buffer
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

}

Output::~Output()
{
}

RTC::ReturnCode_t Output::onInitialize()
{
	std::cout << "--------------------------------------------------" << std::endl;
	std::cout << " Module Name: OutputComponent" << std::endl;
	std::cout << " Vendor  : Toshihiro MIURA" << std::endl;
	std::cout << "           SIT Mizukawa Lab." << std::endl;
	std::cout << "--------------------------------------------------" << std::endl;
	return RTC::RTC_OK;
}

RTC::ReturnCode_t Output::onExecute(RTC::UniqueId ec_id)
{
	char cStr[50];
	if (m_SWIn.isNew())
	{
		m_SWIn.read();
		sprintf(cStr, "SW(8bit, Hex) = %02X", m_SW.data);
		std::cout << cStr << std::endl;
	}
	if (m_DACIn.isNew())
	{
		m_DACIn.read();
		sprintf(cStr, "D/A(16bit, Dec) = %d", m_DAC.data);
		std::cout << cStr << std::endl;
	}
  return RTC::RTC_OK;
}


extern "C"
{
 
  void OutputInit(RTC::Manager* manager)
  {
    RTC::Properties profile(output_spec);
    manager->registerFactory(profile,
                             RTC::Create<Output>,
                             RTC::Delete<Output>);
  }
  
};


