// -*- C++ -*-
/*!
 * @file  ReceivedMessageTest.cpp
 * @brief ReceiveMessageTestComponent
 * @date $Date$
 *
 * $Id$
 */

#include "ReceivedMessageTest.h"

// Module specification
// <rtc-template block="module_spec">
static const char* receivemessagetest_spec[] =
  {
    "implementation_id", "ReceivedMessageTest",
    "type_name",         "ReceivedMessageTest",
    "description",       "ReceiveMessageTestComponent",
    "version",           "1.0.0",
    "vendor",            "ToshihiroMiura_ShibauraHRI",
    "category",          "TestComp",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "10",
    "language",          "C++",
    "lang_type",         "compile",
    ""
  };
// </rtc-template>

ReceivedMessageTest::ReceivedMessageTest(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_RxMessageIn("RxMessage", m_RxMessage),
    
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("RxMessage", m_RxMessageIn);
  
  // Set OutPort buffer
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

}

ReceivedMessageTest::~ReceivedMessageTest()
{
}


CANRxMessageFrame ReceivedMessageTest::ClassifyRxMessage(TimedOctetSeq RxMessage)
{
	int i, iSize;
	CANRxMessageFrame CANRxMessage;
	memset(&CANRxMessage, 0, sizeof(CANRxMessage));
	iSize = RxMessage.data.length();

	for (i = 0; i < 4; i++)
	{
		CANRxMessage.ID = CANRxMessage.ID + (unsigned long)RxMessage.data[3 - i];
		if (i < 3)
			CANRxMessage.ID = CANRxMessage.ID << 8;
	}
	CANRxMessage.DLC = (BYTE)RxMessage.data[i];
	for (i++; i < CANRxMessage.DLC + 5; i++)
		CANRxMessage.Data[i - 5].ByteAccess = (BYTE)RxMessage.data[i];
	if (iSize > CANRxMessage.DLC + 5)
		CANRxMessage.Type.ByteAccess = (BYTE)RxMessage.data[i];
	else
		CANRxMessage.Type.ByteAccess = 0;
	if (iSize == CANRxMessage.DLC +14)
	{
		for (i = (CANRxMessage.DLC + 13); i > (CANRxMessage.DLC + 5); i--)
		{
			CANRxMessage.TimeStamp = CANRxMessage.TimeStamp + RxMessage.data[i];
			if (i > CANRxMessage.DLC + 6)
				CANRxMessage.TimeStamp = CANRxMessage.TimeStamp << 8;
		}
	}
	else
		CANRxMessage.TimeStamp = 0;
	return CANRxMessage;
}


RTC::ReturnCode_t ReceivedMessageTest::onInitialize()
{
	std::cout << "--------------------------------------------------" << std::endl;
	std::cout << " Module Name: ReceivedMessageTestComponent" << std::endl;
	std::cout << " Vendor  : Toshihiro MIURA" << std::endl;
	std::cout << "           SIT Mizukawa Lab." << std::endl;
	std::cout << "--------------------------------------------------" << std::endl;
	return RTC::RTC_OK;
}

RTC::ReturnCode_t ReceivedMessageTest::onExecute(RTC::UniqueId ec_id)
{
	if (m_RxMessageIn.isNew())
	{
		CANRxMessageFrame CANRxMessageTmp;
		memset(&CANRxMessageTmp, 0, sizeof(CANRxMessageTmp));
		m_RxMessageIn.read();
		CANRxMessageTmp = ClassifyRxMessage(m_RxMessage);
		std::cout << "ID:" << CANRxMessageTmp.ID << " DLC:" << (unsigned int)CANRxMessageTmp.DLC << " Data:";
		for (int i = 0; i < (int)CANRxMessageTmp.DLC; i++)
			std::cout << (int)CANRxMessageTmp.Data[i].ByteAccess << ' ';
		std::cout << "Tx/Rx:" << (int)CANRxMessageTmp.Type.TxRx;
		std::cout << " Ch:" << (int)CANRxMessageTmp.Type.Channel;
		std::cout << " TimeStamp[usec]:" << CANRxMessageTmp.TimeStamp << std::endl;
	}
  return RTC::RTC_OK;
}


extern "C"
{
 
  void ReceiveMessageTestInit(RTC::Manager* manager)
  {
    RTC::Properties profile(receivemessagetest_spec);
    manager->registerFactory(profile,
                             RTC::Create<ReceivedMessageTest>,
                             RTC::Delete<ReceivedMessageTest>);
  }
  
};


