// -*- C++ -*-
/*!
 * @file  ReceivedMessageTest.h
 * @brief ReceiveMessageTestComponent
 * @date  $Date$
 *
 * $Id$
 */

#ifndef RECEIVEMESSAGETEST_H
#define RECEIVEMESSAGETEST_H

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>

struct MessageType{
	union{
		unsigned char ByteAccess;
		struct{
		unsigned char Channel:4;								// Channel
		unsigned char TxRx:1;											// MMf
		unsigned char SpareData:3;								// gpĂȂf[^
		};
	};
};

struct MessageData{
	union{
		unsigned char ByteAccess;												// Byte Access
		struct {														// Bit Access
			bool B0:1;											// 0Bit
			bool B1:1;											// 1Bit
			bool B2:1;											// 2Bit
			bool B3:1;											// 3Bit
			bool B4:1;											// 4Bit
			bool B5:1;											// 5Bit
			bool B6:1;											// 6Bit
			bool B7:1;											// 7Bit
		} BitAccess;
	};
};

struct CANRxMessageFrame{
	unsigned long ID;
	unsigned char DLC;
	MessageData Data[8];
	MessageType Type;
	ULONG64 TimeStamp;
};

// Service implementation headers
// <rtc-template block="service_impl_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">

// </rtc-template>

using namespace RTC;

class ReceivedMessageTest
  : public RTC::DataFlowComponentBase
{
 public:
  ReceivedMessageTest(RTC::Manager* manager);
  ~ReceivedMessageTest();
  CANRxMessageFrame ClassifyRxMessage(TimedOctetSeq RxMessage);

  // <rtc-template block="public_attribute">
  
  // </rtc-template>

  // <rtc-template block="public_operation">
  
  // </rtc-template>

  // The initialize action (on CREATED->ALIVE transition)
  // formaer rtc_init_entry() 
  virtual RTC::ReturnCode_t onInitialize();

  // The finalize action (on ALIVE->END transition)
  // formaer rtc_exiting_entry()
  // virtual RTC::ReturnCode_t onFinalize();

  // The startup action when ExecutionContext startup
  // former rtc_starting_entry()
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  // The shutdown action when ExecutionContext stop
  // former rtc_stopping_entry()
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  // The activated action (Active state entry action)
  // former rtc_active_entry()
  // virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  // The deactivated action (Active state exit action)
  // former rtc_active_exit()
  // virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  // The execution action that is invoked periodically
  // former rtc_active_do()
  virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  // The aborting action when main logic error occurred.
  // former rtc_aborting_entry()
  // virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  // The error action in ERROR state
  // former rtc_error_do()
  // virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  // The reset action that is invoked resetting
  // This is same but different the former rtc_init_entry()
  // virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  // The state update action that is invoked after onExecute() action
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  // The action that is invoked when execution context's rate is changed
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // <rtc-template block="protected_attribute">
  
  // </rtc-template>

  // <rtc-template block="protected_operation">
  
  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  TimedOctetSeq m_RxMessage;
  InPort<TimedOctetSeq, RTC::RingBuffer> m_RxMessageIn;
//  InPort<TimedOctetSeq> m_RxMessageIn;
  
  // </rtc-template>


  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  
  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  
  // </rtc-template>

 private:
  int dummy;
  // <rtc-template block="private_attribute">
  
  // </rtc-template>

  // <rtc-template block="private_operation">
  
  // </rtc-template>

};


extern "C"
{
  void ReceiveMessageTestInit(RTC::Manager* manager);
};

#endif // RECEIVEMESSAGETEST_H
